/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2008  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS version: 3.60d                                         *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSInit_91362.c   (for Fujitsu FR 91362)
Purpose : Initializes and handles the hardware for embOS as far
          as required by embOS.
          Feel free to modify this file acc. to your
          target system.
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "mb91362.h"
#include "RTOS.h"

/*********************************************************************
*
*       Configuration
*
**********************************************************************
*/

/*********************************************************************
*
*       Clock frequency settings
*/

#ifndef   OS_FSYS                   /* CPU main clock frequency     */
  #define OS_FSYS 48000000L         /* depends on crystal and PLL   */
#endif

#ifndef   OS_PCLK_TIMER             /* Peripheral clock for timer   */ 
  #define OS_PCLK_TIMER 16000000L   /* Depends on startup and       */
#endif                              /* PLL init function            */

#ifndef   OS_PCLK_UART              /* Peripheral clock for UART    */
  #define OS_PCLK_UART  16000000L   /* Depends on startup and       */
#endif                              /* PLL init function            */
  
/*********************************************************************
*
*       UART settings for embOSView
*       If you do not want (or can not due to hardware limitations)
*       to dedicate a UART to embOSView, please define it to be -1
*       Currently the standard code enables UART 0 per default
*       and supports UART0 and UART1
*/

#ifndef   OS_UART
  #define OS_UART 0
#endif    
#ifndef   OS_BAUDRATE
  #define OS_BAUDRATE 38400L
#endif

#define OS_UART_USED ((OS_UART == 0) || (OS_UART == 1))

/*********************************************************************
*
*       OS_InitHW
*
**********************************************************************
*/

#define OS_TIMER_PRESCALE  (2)     /* Use prescaler 2 per default   */
#define OS_TIMER_FREQUENCY (1000)  /* Default: 1000 ticks/second    */
#define OS_TIMER_PERIODE   (OS_PCLK_TIMER/OS_TIMER_PRESCALE/OS_TIMER_FREQUENCY)  

void OS_InitHW(void) {
  /*  Initialize system timer with 1ms resolution                   */
  /*  Reload timer 0 is used. If any other timer should be used,    */
  /*  modify this init routine and also modify                      */
  /*  Interrupt vector tables (intvecs.inc)                         */
  ICR08  = 31;                   /* First disable interrupt         */
  TMRLR0 = OS_TIMER_PERIODE - 1; /* Initialize calculated reload    */
  TMCSR0 = (0<<10)               /* Clock source: PClk divided by 2 */
          |(1<<4)                /* Reload Mode                     */
          |(1<<3)                /* Interrupt enable                */
          |(1<<1)                /* Count enable                    */
          |(1<<0);               /* Trigger to start timer          */
  ICR08  = 30;                   /* Enable interrupt, lowest prio.  */
  OS_COM_Init();                 /* Initialize comm. to embOSView   */
}

/*********************************************************************
*
*       Idle Loop OS_Idle()
*
**********************************************************************

 Please note:
 This is basically the "core" of the idle loop.
 This core loop can be changed, but:
 The idle loop does not have a stack of its own, therefore no
 functionality should be implemented that relies on the stack
 to be preserved. However, a simple program loop can be programmed
 (like toggeling an output or incrementing a counter)
*/
void OS_Idle(void) {      /* Idle loop: No task is ready to exec */
  for (;;);
}

/*********************************************************************
*
*       Get time [cycles]
*
**********************************************************************

  This routine is required for task-info via embOS Viewer
  or high precision time measurment functions.
  It returns the system time in clock cycles.
*/
OS_U32 OS_GetTime_Cycles(void) {
  OS_U32 time = OS_Time+1;
  unsigned int t_cnt = TMR0;
  /* Check if timer interrupt pending ... */
  if (TMCSR0_UF) {
    t_cnt = TMR0;
    time++;
  }
  return OS_TIMER_PERIODE * time - t_cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us
*
**********************************************************************

Purpose:
   Convert Cycles into micro seconds.

   If your clock frequency is not a multiple of 1 MHz, you may have to
   modify this routine in order to get proper diagonstics.
Please note:
   This routine is required for profiling or high resolution time
   measurement. It does not affect operation of embOS.
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  return Cycles/(OS_PCLK_TIMER/OS_TIMER_PRESCALE/1000000);
}

/*********************************************************************
*
*       OS_ISR_Tick (embOS timer interrupt routine)
*
**********************************************************************
*/
__interrupt void OS_ISR_Tick (void) {
  TMCSR0 &= ~(1 << 2);               /* Reset Timer Underflow flag (clear Int) */
  OS_EnterNestableInterrupt();       /* Re-enable interrupts        */
  OS_HandleTick();                   /* embOS tick handler          */
  OS_LeaveNestableInterrupt();       /* Task switch may occur here  */
}

/*********************************************************************
*
*       Communication to embOSView via UART
*
**********************************************************************
*/

#if OS_UART_USED
  #if (OS_UART == 0)            /* Assign register names to UART 0  */
    #define  UTIMCx_UCC1 UTIMC0_UCC1
    #define  UTIMRx      UTIMR0
    #define  SCRx_CL     SCR0_CL
    #define  SCRx_SBL    SCR0_SBL
    #define  SCRx_PEN    SCR0_PEN
    #define  SCRx_REC    SCR0_REC
    #define  PFRQ_PFQ_RX PFRQ_PFQ0
    #define  PFRQ_PFQ_TX PFRQ_PFQ1
    #define  SMRx_MD0    SMR0_MD0
    #define  SMRx_MD1    SMR0_MD1
    #define  SMRx_CS0    SMR0_CS0
    #define  SCRx_RXE    SCR0_RXE
    #define  SCRx_TXE    SCR0_TXE
    #define  UTIMCx_UTST UTIMC0_UTST
    #define  SSRx_RIE    SSR0_RIE
    #define  ICR_RX      ICR37
    #define  ICR_TX      ICR38
    #define  SODRx       SODR0
    #define  SSRx_TIE    SSR0_TIE
    #define  SIDRx       SIDR0
    #define  SSRx        SSR0
    #define  SCRx_REC    SCR0_REC
  #else                           /* Assign register names to UART 1  */
    #define  UTIMCx_UCC1 UTIMC1_UCC1
    #define  UTIMRx      UTIMR1
    #define  SCRx_CL     SCR1_CL
    #define  SCRx_SBL    SCR1_SBL
    #define  SCRx_PEN    SCR1_PEN
    #define  SCRx_REC    SCR1_REC
    #define  PFRQ_PFQ_RX PFRQ_PFQ0
    #define  PFRQ_PFQ_TX PFRQ_PFQ1
    #define  SMRx_MD0    SMR1_MD0
    #define  SMRx_MD1    SMR1_MD1
    #define  SMRx_CS0    SMR1_CS0
    #define  SCRx_RXE    SCR1_RXE
    #define  SCRx_TXE    SCR1_TXE
    #define  UTIMCx_UTST UTIMC1_UTST
    #define  SSRx_RIE    SSR1_RIE
    #define  ICR_RX      ICR39
    #define  ICR_TX      ICR40
    #define  SODRx       SODR1
    #define  SSRx_TIE    SSR1_TIE
    #define  SIDRx       SIDR1
    #define  SSRx        SSR1
    #define  SCRx_REC    SCR1_REC
  #endif

  #define BRG_DIV ((OS_PCLK_UART / 32 / OS_BAUDRATE) - 1)
  #if (OS_BAUDRATE != 38400 && OS_BAUDRATE != 19200 && OS_BAUDRATE != 9600)
    #error Baudrate OS_BAUDRATE setting no supported !!!
  #endif

/*********************************************************************
*
*       OS_COM_Init()
*
*       Initialize UART for embOSView
*       data format 8N1
*/
void OS_COM_Init(void) {
  UTIMCx_UCC1 = 0;          /* Setup baudrate generator */
  UTIMRx      = BRG_DIV;
  SCRx_CL     = 1;          /* 8 Databits               */
  SCRx_SBL    = 0;          /* 1 Stopbit                */
  SCRx_PEN    = 0;          /* no parity                */
  SCRx_REC    = 0;          /* Clear all errors         */
  PFRQ_PFQ_RX = 1;          /* enable Serial Input Pin  */
  PFRQ_PFQ_TX = 1;          /* enable Serial Output Pin */
  SMRx_MD0    = 0;          /* async normal mode        */
  SMRx_MD1    = 0;
  SMRx_CS0    = 0;          /* select U-timer           */
  SCRx_RXE    = 1;          /* Receive enable           */
  SCRx_TXE    = 1;          /* Transmit enable          */
  UTIMCx_UTST = 1;          /* Start U-Timer            */
  SSRx_RIE    = 1;
  ICR37       = 30;         /* Rx int enable, low prio. */
  ICR38       = 30;         /* Tx int enable, low prio. */
}

/*********************************************************************
*
*       OS_COM_Send1() Sends one byte to embOSView
*
*       NEVER call this function from your application
*/
void OS_COM_Send1(unsigned char c) {
  SODRx = c;
  SSRx_TIE = 1;
}

/*********************************************************************
*
*       OS_ISR_rx() Rx interrupt handler for UART
*/
__interrupt void OS_COM_ISR_rx (void) {
  int Data;
  OS_EnterNestableInterrupt();
  Data = SIDRx;
  if (SSRx & 0xE0) {   /* Check if errors occurred    */
    SCRx_REC = 0;      /* Reset error, ignore data    */
  } else {
    OS_OnRx(Data);     /* Process data                */
  }
  OS_LeaveNestableInterrupt();
}

/*********************************************************************
*
*       OS_ISR_tx() Tx interrupt handler for UART
*/
__interrupt void OS_COM_ISR_tx (void) {
  OS_EnterInterrupt();
  if (OS_OnTx()) {    /*  No more characters to send */
    SSRx_TIE = 0;     /*  => disable Tx interrupt    */
  }
  OS_LeaveInterrupt();
}

#else         /* UART not used for embOSView, using dummies */
  __interrupt void OS_COM_ISR_rx (void) { /* May be removed */
  }
  __interrupt void OS_COM_ISR_tx (void) { /* May be removed */
  }

  void OS_COM_Init(void) {}
  void OS_COM_Send1(OS_U8 c) {
    OS_USEPARA(c);
    OS_COM_ClearTxActive();  /* let OS know that transmitter is not busy */
  }
#endif /* OS_UART_USED */ 

/****** EOF *********************************************************/
