;*******************************************************
;    MB91360 Series startup routine V3.8
;    ===================================
;
;   Startup-Routine for MB91F360 series devices. Initializes 
;   the device after reset and prepares C-operation.
;   Features :
;   - defines all IO variables (includes MB9136x.H)
;   - declares memory sections IO,STACK,DATA,CODE,INTVEC
;   - initializes STACK and DATA area (internal RAM)
;   - defines all interrupt-vectors (includes INTVECS.INC)
;   - sets clock for CPU, Resources, ext.Bus and CAN (4MHz only)
;   - configures clock modulator, depending on the PLL setting
;   - starts user code in "main()"
;
;   Startaddress is 0F:4000 (fixed reset entry)
;   Table Base (location of Interrupt-Vectors) is 0F:FC00
;
;   NOTE : This software is subject to the rules of our standard      
;   DISCLAIMER, that is delivered with our SW-tools (CD-ROM "Micros" 
;   or see our webpage "www.fujitsu-fme.com").
;   (c) Fujitsu Microelectronics Europe GmbH 2002
;
;   Version 3.1
;   MM 02/05/00
;   ME 02/05/01         add ABORT function from MB91F361 
;
;   Version 3.2         for MB91F362 Series
;   MST13/02/02         add NOPS instructions in ABORT function
;                       (checker tool)
;   Version 3.3         for all members of the MB91F360 Series
;   DFi 28.Oct.2002     added smooth pll start and clock modulator
;                       new intvecs.inc (location of table is defined
;                       by the linker)
;
;   Version 3.4         for all members of the MB91F360 Series
;   DFi 18.Dec.2002     added MB91376,
;                       corrected #if DEVICE==91364 && ABORT==1 statement
;                       added CAN memory area selection 10:0000/20:0000
;
;   Version 3.5         for all members of the MB91F360 Series
;   DFi 21.Jan.2003     added MB91376 flash workaround
;
;   Version 3.6         for all members of the MB91F360 Series
;   MSt 12.Mar.2003     added MB91365/367/368 device selection
;
;   Version 3.7         
;   DFi 03.Sept.2003    added comments for clock domains CLKT, CLKP, CLKB
;
;   Version 3.8         
;   JRo 11.Sept.2003    bugfix: location of fixed reset vector
;                       for simulator corrected
;
;   Version 3.9         added _STREAM_INIT function call
;   DFi 06.Oct.2003     
;
;   Version 3.10        - workaround for MB91376 encapsulated
;   DFi 08.Oct.2003       in #if device==91376,
;                       - added C++ constructor calls
;                       - section definition of flash control register
;
;
;*******************************************************

;*******************************************************
;**
;**     This startup routine, originally delivered
;**     as sample from Fujitsu, was modified 
;**     by Segger Microcontroller Systeme GmbH
;**     to be used with embOS
;**
;**     The following modifications were made:
;**
;**     1. #define SIMULATOR modified to enable definition
;**        as assembler tool option
;**
;**     2. stack base and top addresses exported to enable
;**        embOS stack check functions (required)
;**
;**     3. Reset entry for SIMULATOR removed, as this
;**        did not link.
;**
;**     4. Interrupt priority register always initialized
;**        regardless of SIMULATOR and ABORT setting
;**
;*******************************************************

; Global settings :

#define DEVICE 91364        // select device number
                            // 91362 = MB91362
                            // 91364 = MB91364
                            // 91365 = MB91365
                            // 91366 = MB91366
                            // 91367 = MB91367
                            // 91368 = MB91368
                            // 91369 = MB91369
                            // 91376 = MB91376

#define PLL 48              // select PLL-Clock (16,32,48 or 64 MHz)
                            // PLL     core    ext. bus   resource bus
                            //         CLKB    CLKT       CLKP
                            // ---------------------------------------
                            // 16MHz   16MHz   16MHz      16MHz
                            // 32MHz   32MHz   16MHz      16MHz
                            // 48MHz   48MHz   16MHz      16MHz
                            // 64MHz   64MHz   16MHz      16MHz

#define CLOMO       0       // enable(1) or disable(0) clock modulator (only if PLL-Clock<64MHz)
                            // please refer to the hardware manual if you enable clock modulation
                            // modulation degree is set, dependant on the PLL-Clock

;*******************************************************
;****   Modification 1:
;****   Allow setting of SIMULATOR as assembler tool option.
;****   This enables switching between Simulator or target
;****   without the need of modifying this startup source

#ifndef SIMULATOR           // May be defined as project setting
  #define SIMULATOR 0       // set to 1 when using the simulator
#endif  

;****   End of Modification 1 **************************
;*******************************************************

#define ABORT       0       // enable(1) or disable(0) USER0-Button on Starterkit MB91360 for ABORT
#define I_RAM       0       // enable(1) or disable(0) copy code from ROM to I_RAM
#define STREAM_INIT 0       // enable(1) or disable(0) call of _stream_init function


#define SSTACK_SIZE 0x200   // define system (interrupt) stack size: 512 bytes
#define USTACK_SIZE 0x200   // define user stack size: 512 bytes

; check global settings

#if DEVICE==91364
  #if ABORT==1
    #error abort=1: The MB91364 starterkit does not feature an abort button, please set abort to 0
  #endif

  #if I_RAM==1
      #error i_ram=1: MB91364 does not feature IRAM, please set i_ram to 0
  #endif
#endif

#if CLOMO==1 && PLL==64
    #error pll=64MHz: If the clock modulator is enabled, the pll frequency must be below 64 MHz
#endif


;*******************************************************
;   IO Symbol definition
;*******************************************************

#define __IO_DEFINE         // Activate IO assembler definitions

#if DEVICE==91362
  #include "MB91362.H"
#elif DEVICE==91364
  #include "MB91364.H"
#elif DEVICE==91365
  #include "MB91365.H"  
#elif DEVICE==91366
  #include "MB91366.H"
#elif DEVICE==91367
  #include "MB91367.H"
#elif DEVICE==91368
  #include "MB91368.H"
#elif DEVICE==91369
  #include "MB91369.H"
#elif DEVICE==91376
  #include "MB91376.H"
#endif

        .export start
        .export __exit
        .export uninitial_int
        .import _main
        .import _exit
        .import __stream_init
        .import _RAM_INIT
        .import _ROM_INIT
        .import _switch_pll
#if I_RAM 
        .import _RAM_IRAM
        .import _ROM_IRAM
#endif

;*******************************************************
;   Declare STACK (RAM) area and reserve space
;*******************************************************

;*******************************************************
;****   Modification 2:
;****   Stack addresses exported to enable embOS stack
;****   check function

        .EXPORT __systemstack, __systemstack_top
        .EXPORT __userstack, __userstack_top

        .section        STACK, stack, align=4
__systemstack
        .res.b          SSTACK_SIZE
__systemstack_top:

__userstack:
        .res.b          USTACK_SIZE
__userstack_top:        

;****   End of Modification 2 **************************
;*******************************************************

;*******************************************************
;   Interrupt-Vector definition 
;*******************************************************

#include "intvecs.inc"      

;*******************************************************
;****   Modification 3:
;****   The following reset vector definition was removed
;****   by Segger Microcontroller Systeme GmbH, as this
;****   does not link, when SIMULATOR is used

#if 0

;*******************************************************
;   Fixed Reset Vector (Simulator information only)
;*******************************************************

#if SIMULATOR
        .ORG   0x1FFFFC
        .DATA   start           ; reset-vector 
#endif
#endif

;****   End of Modification 3 **************************
;*******************************************************

;*******************************************************
;   Declare IO-Sections and reserve space
;*******************************************************

        .SECTION        IO, DATA, locate=0x0
        .RES.B  0x1028

;********* section for flash control registers *********
        .SECTION        FCR, DATA, locate=0x7000
        .RES.B  0x8

#if DEVICE==91376
        .SECTION        CAN, DATA, locate=0x200000
#else
        .SECTION        CAN, DATA, locate=0x100000
#endif

        .RES.B  0x800

;*******************************************************
;   Declare user sections
;*******************************************************

        .section        DATA,  data,  align=4
        .section        INIT,  data,  align=4
        .section        IRAM,  code,  align=4
        .section        CONST, const, align=4
        .section        EXT_CTOR_DTOR, const, align=4  ;C++ constructors

;*******************************************************


;*******************************************************
;   Start of Init-Routine 
;   =====================
;
;   Operation will start here after reset
;   (fixed Reset entry from BootROM -> 0F:4000)
;*******************************************************

        .section        START,  code,  align=4
start:
        nop     ; Reset-entry

;*************************************************************************
;       Flash initialization
;*************************************************************************

       LDI  #FMCS,R12
       LDI  #0x60, R0          ; Facc and fix R-Vector on ; BIROM and LPM off
       STB  R0,@R12

       ; FMWT is set by the switch_pll function

;*******************************************************
;       Set Stack Pointer and Table Base Register
;*******************************************************

        ldi     #__systemstack_top, sp ; set system (Interrupt) Stack Pointer
        ldi     #INTVECT,R0            ; set Table Base
        mov     r0,tbr

        orccr   #(1<<5)                ; select user stack
        ldi     #__userstack_top, sp   ; set Stack Pointer


#if !SIMULATOR

;*******************************************************
;       Clock Settings
;*******************************************************
; smooth PLL start
;
; FMWT register settings
;
; PLL  CLOMO OFF ---------   CLOMO ON ----------
; MHz  91362 others  91376   91362 others  91376
; 64   0x13  0x13    0x13    -     -       -
; 48   0x13  0x02    0x1A    0x13  0x13    0x13 
; 32   0x02  0x09    0x09    0x13  0x02    0x1A
; 16   0x01  0x01    0x01    0x01  0x01    0x09



; prepare configuration main PLL :



#if   PLL == 16   ; 16 MHz
  LDI:8    #0x30,R4
  LDI:8    #0x00,R5
  LDI:8    #0x00,R6
  #if CLOMO == 1             ; clock modulator enabled
    #if  DEVICE == 91376   ; Flash macro in MB91376
      LDI:8    #0x09,R7  ; FMWT: FATDIN_high/wait_cycles: 0.5/1
    #else
      LDI:8    #0x01,R7  ; FMWT: FATDIN_high/wait_cycles: 0.5/1
    #endif
  #else 
    LDI:8    #0x01,R7      ; FMWT: FATDIN_high/wait_cycles: 0.5/1
  #endif



#elif PLL == 32   ; 32 MHz
  LDI:8    #0x40,R4
  LDI:8    #0x01,R5
  LDI:8    #0x10,R6
  #if CLOMO == 1             ; clock modulator enabled
    #if  DEVICE == 91362   ; normal Flash macro in MB91362
      LDI:8    #0x13,R7  ; FACC_low/wait_cycles: 1/3
    #elif  DEVICE == 91376 ; Flash macro in MB91376
      LDI:8    #0x1A,R7  ; ATDIN_high/wait_cycles: 1/2, EQINH
    #else                  ; fast Flash macro in other devices
      LDI:8    #0x02,R7  ; ATDIN_high/wait_cycles: 0.5/2, EQINH
    #endif
  #else
    #if  DEVICE == 91362   ; normal Flash macro in MB91362
      LDI:8    #0x02,R7  ; FMWT: FACC_low/wait_cycles: 0.5/2
    #else                  ; fast Flash macro in other devices
      LDI:8    #0x09,R7  ; FMWT: ATDIN_high/wait_cycles: 0.5/1, EQINH
    #endif
  #endif



#elif PLL == 48   ; 48 MHz
  LDI:8    #0x60,R4
  LDI:8    #0x02,R5
  LDI:8    #0x20,R6
  #if  DEVICE == 91362          ; normal Flash macro in MB91362
    LDI:8    #0x13,R7         ; FACC_low/wait_cycles: 1/3
  #else                         ; fast Flash macro in other devices
    #if CLOMO == 1            ; clock modulator enabled
      LDI:8    #0x13,R7     ; FMWT: ATDIN_high/wait_cycles: 1/3
    #else
      #if DEVICE == 91362   ; normal Flash macro in MB91362
        LDI:8    #0x13,R7 ; ATDIN_high/wait_cycles: 1/2, EQINH
      #elif DEVICE == 91376 ; Flash macro in MB91376
        LDI:8    #0x1A,R7 ; ATDIN_high/wait_cycles: 1/2, EQINH
      #else                 ; fast Flash macro in other devices
        LDI:8    #0x02,R7 ; FMWT: ATDIN_high/wait_cycles: 0.5/2
      #endif
    #endif
  #endif


#elif PLL == 64   ; 64 MHz
  LDI:8    #0x70,R4
  LDI:8    #0x03,R5
  LDI:8    #0x30,R6
  LDI:8    #0x13,R7          ; FMWT: ATDIN_high/wait_cycles: 1/3
#endif



  LDI:32   #_switch_pll,R12
  CALL     @R12
    



;   set CAN clock :

    ldi #CMCR, R12  ; CAN clock :
#if   PLL == 64
    ldi #0x0180, r1 ; CAN = PLL/4 
                    ; PLL=64MHz -> CAN-Clk=16MHz
#else
    ldi #0x0080, r1 ; CAN = PLL/2
                    ; PLL=16Mhz -> CAN-Clk=8MHz
                    ; PLL=32Mhz -> CAN-Clk=16MHz
                    ; PLL=48Mhz -> CAN-Clk=24MHz
#endif
    sth r1,@r12

; enable interrupts
  orccr #0x10
  
; end SMOOTH_PLL

;********************************************************
; Clock Modulator
;********************************************************
#if CLOMO
; PLL=16: resolution: high,   degree: 1
; PLL=32: resolution: medium, degree: 2
; PLL=48: resolution: low,    degree: 2
; PLL=64: modulation not allowed 

    ;*****************************************
  ; random number generator control register
  ; common for all PLL settings
    LDI  #CMLT0,R4
    LDI  #CMLT1,R5
    LDI  #CMLT2,R6
    LDI  #CMLT3,R7

    LDI:20  #0xF802,R0

    STH R0, @R4
    STH R0, @R5
    STH R0, @R6
    STH R0, @R7

    ;*****************************************
  ; frequency resolution setting register
    LDI  #CMLS0,R4
    LDI  #CMLS1,R5
    LDI  #CMLS2,R6
    LDI  #CMLS3,R7

    LDI:20  #0xF800,R0
    LDI:20  #0xFF04,R1
  #if   PLL == 16         ; resolution: high
    LDI:20  #0xF813,R2
    LDI:20  #0xFF84,R3
  #elif PLL == 32         ; resolution: medium
    LDI:20  #0xF813,R2
    LDI:20  #0x0000,R3
  #elif PLL == 48         ; resolution: low
    LDI:20  #0x0000,R2
    LDI:20  #0x0000,R3
  #endif

    STH R0, @R4
    STH R1, @R5
    STH R2, @R6
    STH R3, @R7


    ;*****************************************
  ; parameter register
    LDI  #CMPR,R4

  #if   PLL == 16         ; resolution: high,   degree: 1
    LDI:20  #0x0981,R0
  #elif PLL == 32         ; resolution: medium, degree: 2
    LDI:20  #0x0A82,R0
  #elif PLL == 48         ; resolution: low,    degree: 2
    LDI:20  #0x0E82,R0
  #endif

    STH R0, @R4

    ;*****************************************
  ; control register
    LDI     #CMCR,R4

    LDUH    @R4,R0
    LDI:20  #0xFF80,R1  ; MSEL=0, MTST=0
    LDI:20  #0x0033,R2  ; enable clock modulator
    AND     R1,R0
    OR      R2,R0

    STH     R0, @R4

#endif

;********************************************************
;   Enable external INT0 for USER0-Button (ABORT)
;********************************************************

#if ABORT

  LDI:8    #0x10,R0   ; set interrupt control register  ICR00 = 16
  LDI:32   #_icr00,R12
  STB      R0,@R12

  LDI:8    #0x1F,R1   ; disable all other interrupts (ICR01..ICR47 = 31)
  LDI:32   #_icr01,R4 
     MOV      R4, R5
fillICRs:
  STB      R1,@R5
  ADD      #1,R4
  LDI:20   #0x470,R0
  ADD      #1,R5
  CMP      R0,R4
  BLT      fillICRs

  LDI:32   #0x41,R11
  LDI:32   #0x42,R10
  BANDL    #0xE,@R11  ; enable interrupt #0
  LDUH     @R10,R0  ; set lower 2 bits of ELVR to 1
  LDI:20   #0x3,R1    ; (interrupt will be signaled at button down)
  OR       R1,R0
  STH      R0,@R10

  LDI:32   #0x40,R0 ; clear old interrupts
  BANDL    #0xE,@R0
  BORL     #1,@R11  ; enable INT0 now

;*******************************************************
;****   Modification 4:
;****   To enable interrupt simulation when using SIMULATOR
;****   The following two lines were commented out
;****   and moved by Segger Microcontroller Systeme GmbH
;****   (see comments below the following two #endif)
 
;        ORCCR   #0x10      ; enable interrupts
;        STILM   #0x1E      ; set global interrupt level to 30

#endif

;****   Enbale interrupts when ABORT is set only
        ORCCR   #0x10           ; enable interrupts


#endif                          ; end of simulator used 

;****   Always initialize interrupt priority register
        STILM   #0x1F           ; set global interrupt level to 31 (lowest priority)

;****   End of Modification 4 **************************
;*******************************************************

;********************************************************
;   C setup (clears Data RAM and initializes variables)
;********************************************************

;       clear DATA section

        ldi:8   #0, r0

        ldi     #sizeof DATA &~0x3, r1
        ldi     #DATA, r13

        cmp     #0, r1
        beq     data_clr1
data_clr0:
        add2    #-4, r1
        bne:d   data_clr0
        st      r0, @(r13, r1)

data_clr1:
        ldi:8   #sizeof DATA & 0x3, r1
        ldi     #DATA + (sizeof DATA & ~0x3), r13

        cmp     #0, r1
        beq     data_clr_end
data_clr2:
        add2    #-1, r1
        bne:d   data_clr2
        stb     r0, @(r13, r1)
data_clr_end:

;********************************************************
;       copy rom (initialize variables)
;********************************************************

        ldi     #_RAM_INIT, r0
        ldi     #_ROM_INIT, r1
        ldi     #sizeof(INIT), r2
        cmp     #0, r2
        beq:d   copy_rom_end
        ldi     #3, r12
        and     r2, r12
        beq:d   copy_rom2
        mov     r2, r13
        mov     r2, r3
        sub     r12, r3
copy_rom1:
        add     #-1, r13
        ldub    @(r13, r1), r12
        cmp     r3, r13
        bhi:d   copy_rom1
        stb     r12, @(r13, r0)
        cmp     #0, r3
        beq:d   copy_rom_end
copy_rom2:
        add     #-4, r13
        ld      @(r13, r1), r12
        bgt:d   copy_rom2
        st      r12, @(r13, r0)
copy_rom_end:

        
;********************************************************
;       copy code from Flash to I-RAM 
;********************************************************
#if I_RAM
        ldi     #_RAM_IRAM, r0
        ldi     #_ROM_IRAM, r1
        ldi     #sizeof(IRAM), r2
        cmp     #0, r2
        beq:d   copy_iram_end
        ldi     #3, r12
        and     r2, r12
        beq:d   copy_iram2
        mov     r2, r13
        mov     r2, r3
        sub     r12, r3
copy_iram1:
        add     #-1, r13
        ldub    @(r13, r1), r12
        cmp     r3, r13
        bhi:d   copy_iram1
        stb     r12, @(r13, r0)
        cmp     #0, r3
        beq:d   copy_iram_end
copy_iram2:
        add     #-4, r13
        ld      @(r13, r1), r12
        bgt:d   copy_iram2
        st      r12, @(r13, r0)
copy_iram_end:
#endif 



;********************************************************
;       call C++ constructors
;********************************************************

        ldi      #sizeof(EXT_CTOR_DTOR),r0
        cmp      #0,r0
        beq      call_ctor_end
call_ctor:
        ldi:32   #EXT_CTOR_DTOR,r0
        ld       @R0,r12
        call     @r12
call_ctor_end:


;*******************************************************
; MB91F376 flash workaround
;*******************************************************

#if DEVICE==91376

        ldi:20   #0x44511,R12
        ldub     @R12,R0
        ldi:8    #0x57,R1
        cmp      R1,R0
        bne      mb91f376_workaround_end

        ldi:20   #0x07000,R12
        ldi:8    #0x9F,R1
        andb     R1,@R12

mb91f376_workaround_end:

#endif



;*******************************************************
; call of _stream_init() function
;*******************************************************

#if STREAM_INIT
        call32  __stream_init, r12
#endif
       

;*******************************************************

        ldi  #start_main, r12
        jmp  @r12            ; jump to main code


;*******************************************************
; end of startup-routines
;*******************************************************
       
       .section CODE, code, align=4
       
start_main:

        call32  _main, r12  ;   jump to user routine (main)

__exit:
end:
    bra end             ; in case of return from main

;*******************************************************
;   error handler for uninitialized interrupts
;*******************************************************

uninitial_int:
/*##################################################*/
/*                                                  */
/* ERROR : UNINITIALIZED INTERRUPT  !               */
/* An interrupt was called for which no             */
/* interrupt vector is defined !                    */
/* Leave the debugger and correct your              */
/* project (see instructions in INTVECT.ASM) !      */
/*                                                  */
/*##################################################*/
#if !SIMULATOR
        INTE  ; break here
#endif
        bra uninitial_int

;*******************************************************
;   handler for ABORT-Button (USER0)
;*******************************************************

abortbutton:

#if ABORT

  STM0     (R4,R5,R6,R7)
  STM1     (R12,R13)
  STM0     (R0,R1,R2,R3)
  ST       MDH,@-R15
  ST       MDL,@-R15
  ENTER    #004

  LDI:32   #0x040,R0  ; clear interrupt cause
  BANDL    #0xE,@R0

  LEAVE    
  NOP     ; NOP instruction added
  LD       @R15+,MDL
  NOP     ; NOP instruction added
  LD       @R15+,MDH
  LDM0     (R0,R1,R2,R3)
  LDM1     (R12,R13)
  LDM0     (R4,R5,R6,R7)

  INTE          ; break here 

/*##################################################*/
/*                                                  */
/* You pressed ABORT (USER0)                        */
/* on the Starterkit !                              */
/* Execute a single-step to return                  */
/* to the source file or add breakpoints !          */
/*                                                  */
/*##################################################*/

    RETI

#endif

;*******************************************************

    .end    start

/*EOF*/
