/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2008  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS version: 3.60d                                         *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : Rtosinit_91469G.c            (for Fujitsu FR MB91469G)
Purpose : Initializes and handles the hardware for embOS as far
          as required by embOS.
          Feel free to modify this file acc. to your
          target system.
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.H"
#include "mb91469g.h"

/*********************************************************************
*
*       Configuration
*
**********************************************************************
*/

/*********************************************************************
*
*       Clock frequency settings
*/
#ifndef   OS_FSYS                   /* CPU main clock frequency     */
  #define OS_FSYS 64000000L         /* depends on crystal and PLL   */
#endif

#ifndef   OS_PCLK_TIMER             /* Peripheral clock for timer   */
  #define OS_PCLK_TIMER 16000000L   /* Depends on startup and       */
#endif                              /* PLL init function            */

#ifndef   OS_PCLK_UART              /* Peripheral clock for UART    */
  #define OS_PCLK_UART  16000000L   /* Depends on startup and       */
#endif                              /* PLL init function            */

/*********************************************************************
*
*       UART settings for embOSView
*       If you do not want (or can not due to hardware limitations)
*       to dedicate a UART to embOSView, please define it to be -1
*       Currently the standard code enables UART 1 per default
*       UART1 only. Others may be easily added by defines.
*/
#ifndef   OS_UART
  #define OS_UART     (1)
#endif    
#ifndef   OS_BAUDRATE
  #define OS_BAUDRATE (38400L)
#endif

#define OS_UART_USED (OS_UART == 1)

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

#define _TRIGGER_WD() (HWWD_CL = 0)

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitHW
*/
#define OS_TIMER_PRESCALE  (2)     /* Use prescaler 2 per default   */
#define OS_TIMER_FREQUENCY (1000)  /* Default: 1000 ticks/second    */
#define OS_TIMER_PERIODE   (OS_PCLK_TIMER/OS_TIMER_PRESCALE/OS_TIMER_FREQUENCY)  

#define ICR_TIMER   ICR08

void OS_InitHW(void) {
  /*  Initialize system timer with 1ms resolution                   */
  /*  Reload timer 0 is used. If any other timer should be used,    */
  /*  modify this init routine and also modify                      */
  /*  Interrupt vector tables (intvecs.inc)                         */
  ICR_TIMER = 31;                /* First disable interrupt         */
  TMRLR0 = OS_TIMER_PERIODE - 1; /* Initialize calculated reload    */
  TMCSR0 = (0 << 10)             /* Clock source: PClk divided by 2 */
         | (1 << 4)              /* Reload Mode                     */
         | (1 << 3)              /* Interrupt enable                */
         | (1 << 1)              /* Count enable                    */
         | (1 << 0)              /* Trigger to start timer          */
         ;
  ICR_TIMER = 29;                /* Enable interrupt, lowest prio.  */
  OS_COM_Init();                 /* Initialize comm. to embOSView   */
}

/*********************************************************************
*
*       Idle Loop OS_Idle()
*
**********************************************************************

 Please note:
 This is basically the "core" of the idle loop.
 This core loop can be changed, but:
 The idle loop does not have a stack of its own, therefore no
 functionality should be implemented that relies on the stack
 to be preserved. However, a simple program loop can be programmed
 (like toggeling an output or incrementing a counter)
*/
void OS_Idle(void) {      /* Idle loop: No task is ready to exec */
  for (;;) {
  }
}

/*********************************************************************
*
*       Get time [cycles]
*
**********************************************************************

  This routine is required for task-info via embOS Viewer
  or high precision time measurment functions.
  It returns the system time in clock cycles.
*/
OS_U32 OS_GetTime_Cycles(void) {
  OS_U32 time = OS_Time+1;
  unsigned int t_cnt = TMR0;
  /* Check if timer interrupt pending ... */
  if (TMCSR0_UF) {
    t_cnt = TMR0;
    time++;
  }
  return OS_TIMER_PERIODE * time - t_cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us
*
**********************************************************************

Purpose:
   Convert Cycles into micro seconds.

   If your clock frequency is not a multiple of 1 MHz, you may have to
   modify this routine in order to get proper diagonstics.
Please note:
   This routine is required for profiling or high resolution time
   measurement. It does not affect operation of embOS.
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  return Cycles/(OS_PCLK_TIMER/OS_TIMER_PRESCALE/1000000);
}

/*********************************************************************
*
*       OS_ISR_Tick (embOS timer interrupt routine)
*
**********************************************************************
*/
__interrupt void OS_ISR_Tick (void) {
  TMCSR0 &= ~(1 << 2);    /* Reset Timer Underflow flag (clear Int) */
  _TRIGGER_WD();
  OS_EnterNestableInterrupt();
  OS_HandleTick();
  OS_LeaveNestableInterrupt();
}

/*********************************************************************
*
*       Communication to embOSView via UART
*
**********************************************************************
*/

#if OS_UART_USED
  #if (OS_UART == 1)            /* Assign register names to UART 1  */
    #define ICR_UART  (ICR20)
    #define SCRxx     (SCR01)
    #define BGRxx     (BGR01)
    #define SMRxx     (SMR01)
    #define SSRxx     (SSR01)
    #define TDRxx     (TDR01)
    #define RDRxx     (RDR01)
    #define SSRxx_TIE (SSR01_TIE)
    #define SCRxx_CRE (SCR01_CRE)
    #define PFRxx     (PFR21)
    #define PFR_VAL   ((1 << 4) | (1 << 5))
  #else
    #error "Selected UART is not supported."
  #endif

  #define BRG_DIV ((OS_PCLK_UART / OS_BAUDRATE) - 1)

/*********************************************************************
*
*       OS_COM_Init()
*
*       Initialize UART for embOSView
*       data format 8N1
*/
void OS_COM_Init(void) {
  SCRxx  = 0x00
         | (0 << 0)    // Disable Tx
         | (0 << 1)    // Disable Rx
         | (1 << 2)    // Reset all errors
         | (0 << 3)    // AD: Data bit
         | (1 << 4)    // CL: 8bits/ character
         | (0 << 5)    // SBL: 1 Stop bit
         | (0 << 6)    // P: Even parity (dc because parity is not used
         | (0 << 7)    // PEN: Parity disabled
         ;

  BGRxx  = BRG_DIV;    // Setup baudrate generator

  SMRxx  = 0        
         | (1 << 0)    // SOE:  Enable TxD output pin
         | (0 << 1)    // SCKE: No clock output
         | (1 << 2)    // UPCL: Reset UART
         | (1 << 3)    // REST: Restart counter
         | (0 << 4)    // EXT:  Use internal clock
         | (0 << 5)    // OTO:  Use external clock with baudrate generator (dc, because internal clock is used)
         | (0 << 6)    // Mode0: Asynchronous normal
         ;

  SMRxx  = 0           // Setup again, without Reset and Restart command       
         | (1 << 0)    // SOE:  Enable TxD output pin
         | (0 << 1)    // SCKE: No clock output
         | (0 << 2)    // UPCL: Do not Reset UART
         | (0 << 3)    // REST: Do not Restart counter
         | (0 << 4)    // EXT:  Use internal clock
         | (0 << 5)    // OTO:  Use external clock with baudrate generator (dc, because internal clock is used)
         | (0 << 6)    // Mode0: Asynchronous normal
         ;

  SCRxx  = 0x00
         | (1 << 0)    // Disable Tx
         | (1 << 1)    // Disable Rx
         | (1 << 2)    // Reset all errors
         | (0 << 3)    // AD: Data bit
         | (1 << 4)    // CL: 8bits/ character
         | (0 << 5)    // SBL: 1 Stop bit
         | (0 << 6)    // P: Even parity (dc because parity is not used
         | (0 << 7)    // PEN: Parity disabled
         ;

  SSRxx  = 0x00
         | (0 << 0)    // TIE: Initially disable Tx interupts
         | (1 << 1)    // RIE: Initially enable Rx interrupts
         | (0 << 2)    // BDS: Direction, LSB first
         ;             // other bits are read only
          
  PFRxx |= PFR_VAL;    // enable UART: SIN,SOT for async. transfer

  ICR_UART    = 30;                /* Rx/Tx int enable, low prio. */
}

/*********************************************************************
*
*       OS_COM_Send1() Sends one byte to embOSView
*
*       NEVER call this function from your application
*/
void OS_COM_Send1(unsigned char c) {
  TDRxx     = c;
  SSRxx_TIE = 1;
}

/*********************************************************************
*
*       OS_COM_ISR_rx() Rx interrupt handler for UART
*/
__interrupt void OS_COM_ISR_rx (void) { /* May be removed */
  OS_U8  Data;
  
  OS_EnterNestableInterrupt();
  // Always read the data
  Data = RDRxx;
  if (SSRxx & 0xE0) {  /* Any error occurred ?        */
    SCRxx_CRE = 1;     /* Reset error, ignore data    */
    SCRxx_CRE = 0;     /* Reset error, ignore data    */
  } else {
    OS_OnRx(Data);     /* Process data                */
  }
  OS_LeaveNestableInterrupt();
}

/*********************************************************************
*
*       OS_COM_ISR_tx() Tx interrupt handler for UART
*/
__interrupt void OS_COM_ISR_tx (void) { /* May be removed */
  OS_EnterNestableInterrupt();
  if (OS_OnTx()) {  /*  No more characters to send ? */
    SSRxx_TIE = 0;  /*  => disable Tx interrupt      */
  }
  OS_LeaveNestableInterrupt();
}

/********************************************************************/

#else         /* UART not used for embOSView, using dummies */

  void OS_COM_Init(void) {}
  void OS_COM_Send1(OS_U8 c) {
    OS_USEPARA(c);
    OS_COM_ClearTxActive();  /* let OS know that transmitter is not busy */
  }
#endif /* OS_UART_USED */ 

/****** EOF *********************************************************/

